Helm 是 Kubernetes 的包管理器，用于管理 Kubernetes 应用程序。以下是一些常用的 Helm 命令，帮助你在 Kubernetes 集群中更有效地管理应用程序。

### 基本命令

#### 1. 安装 Chart
```sh
helm install <release_name> <chart_name>
```
- `release_name`：指定这次安装的名称。
- `chart_name`：指定要安装的 Chart 名称。

#### 2. 升级 Release
```sh
helm upgrade <release_name> <chart_name>
```
- `release_name`：要升级的 Release 的名称。
- `chart_name`：新的 Chart 名称或路径。

#### 3. 卸载 Release
```sh
helm uninstall <release_name>
```
- `release_name`：要卸载的 Release 的名称。

#### 4. 查看 Release
```sh
helm list
```
- 列出所有已安装的 Release。

#### 5. 查看 Release 状态
```sh
helm status <release_name>
```
- 查看指定 Release 的状态。

#### 6. 查看 Release 历史
```sh
helm history <release_name>
```
- 查看指定 Release 的历史记录。

### Chart 管理

#### 7. 搜索 Chart
```sh
helm search hub <keyword>
```
- 在 Helm Hub 中搜索 Chart。

```sh
helm search repo <keyword>
```
- 在已添加的仓库中搜索 Chart。

#### 8. 添加仓库
```sh
helm repo add <repo_name> <repo_url>
```
- `repo_name`：仓库的名称。
- `repo_url`：仓库的 URL。

#### 9. 更新仓库
```sh
helm repo update
```
- 更新所有已添加仓库的信息。

#### 10. 列出仓库
```sh
helm repo list
```
- 列出所有已添加的仓库。

#### 11. 移除仓库
```sh
helm repo remove <repo_name>
```
- 移除指定的仓库。

### Chart 开发

#### 12. 创建新的 Chart
```sh
helm create <chart_name>
```
- 创建一个新的 Chart 目录结构。

#### 13. 打包 Chart
```sh
helm package <chart_path>
```
- 将指定目录下的 Chart 打包成 `.tgz` 文件。

#### 14. 验证 Chart
```sh
helm lint <chart_path>
```
- 检查 Chart 是否符合规范，有无错误。

### 配置管理

#### 15. 查看默认值
```sh
helm show values <chart_name>
```
- 查看 Chart 的默认值。

#### 16. 使用自定义值安装/升级
```sh
helm install <release_name> <chart_name> -f <custom_values_file>
```
```sh
helm upgrade <release_name> <chart_name> -f <custom_values_file>
```
- 使用指定的自定义值文件进行安装或升级。

### Helm 插件

#### 17. 列出插件
```sh
helm plugin list
```
- 列出所有已安装的 Helm 插件。

#### 18. 安装插件
```sh
helm plugin install <plugin_url>
```
- 从指定 URL 安装插件。

#### 19. 移除插件
```sh
helm plugin uninstall <plugin_name>
```
- 移除指定的插件。

### 其他

#### 20. 获取 Helm 版本
```sh
helm version
```
- 查看当前 Helm 客户端和服务端的版本信息。

### 总结
这些是 Helm 的一些常用命令，掌握这些命令可以帮助你在 Kubernetes 集群中更高效地管理应用程序。Helm 提供了强大且灵活的功能，适用于各种场景的应用管理。