### 和序列相关的内置函数

- list
- tuple
- set
- dict
- string

| 函数        | 功能                                                         |
| :---------- | :----------------------------------------------------------- |
| len()       | 计算序列的长度，即返回序列中包含多少个元素。                 |
| max()       | 找出序列中的最大元素。注意，对序列使用 sum() 函数时，做加和操作的必须都是数字，不能是字符或字符串，否则该函数将抛出异常，因为解释器无法判定是要做连接操作（+ 运算符可以连接两个序列），还是做加和操作。 |
| min()       | 找出序列中的最小元素。                                       |
| list()      | 将序列转换为列表。                                           |
| str()       | 将序列转换为字符串。                                         |
| sum()       | 计算元素和。                                                 |
| sorted()    | 对元素进行排序。                                             |
| reversed()  | 反向序列中的元素。                                           |
| enumerate() | 将序列组合为一个索引序列，多用在 for 循环中。                |

```python
citys = ["jinan", "qingdao", "yantai", "zibo"]
for city in enumerate(citys):
    print(city)
# 输出：    
(0, 'jinan')
(1, 'qingdao')
(2, 'yantai')
(3, 'zibo')
```

```python
citys = ["jinan", "qingdao", "yantai", "zibo"]
for index,city in enumerate(citys):
    print(index,city)
# 输出： 
0 jinan
1 qingdao
2 yantai
3 zibo
```



### 字符串相关内置函数

| 序号 | 方法及描述                                                   |
| :--- | :----------------------------------------------------------- |
| 1    | [capitalize()](https://www.w3cschool.cn/python3/python3-string-capitalize.html) 将字符串的第一个字符转换为大写 |
| 2    | [center(width, fillchar)](https://www.w3cschool.cn/python3/python3-string-center.html)返回一个指定的宽度 width 居中的字符串，fillchar 为填充的字符，默认为空格。 |
| 3    | [count(str, beg= 0,end=len(string))](https://www.w3cschool.cn/python3/python3-string-count.html)返回 str 在 string 里面出现的次数，如果指定 beg 或者 end，则返回指定范围内 str 出现的次数 |
| 4    | [bytes.decode(encoding="utf-8", errors="strict")](https://www.w3cschool.cn/python3/python3-string-decode.html)Python3 中没有 decode 方法，但我们可以使用 bytes 对象的 decode() 方法来解码给定的 bytes 对象，这个 bytes 对象可以由 str.encode() 来编码返回。 |
| 5    | [encode(encoding='UTF-8',errors='strict')](https://www.w3cschool.cn/python3/python3-string-encode.html)以 encoding 指定的编码格式编码字符串，如果出错默认报`ValueError`异常，除非 errors 指定的是'ignore'或者'replace' |
| 6    | [endswith(suffix, beg=0, end=len(string))](https://www.w3cschool.cn/python3/python3-string-endswith.html) 检查字符串是否以指定的字符串结束，如果指定了beg 或 end 则检查指定的范围内是否以指定的字符串结束，如果是，返回 True，否则返回 False. |
| 7    | [expandtabs(tabsize=8)](https://www.w3cschool.cn/python3/python3-string-expandtabs.html)把字符串 string 中的 tab 符号转为空格，tab 符号默认的空格数是 8 。 |
| 8    | [find(str, beg=0, end=len(string))](https://www.w3cschool.cn/python3/python3-string-find.html)检测 str 是否包含在字符串中，如果指定范围 beg 和 end ，则检查是否包含在指定范围内，如果包含，返回开始的索引值，否则返回-1 |
| 9    | [index(str, beg=0, end=len(string))](https://www.w3cschool.cn/python3/python3-string-index.html)跟find()方法一样，只不过如果str不在字符串中会报一个异常。 |
| 10   | [isalnum()](https://www.w3cschool.cn/python3/python3-string-isalnum.html)如果字符串至少有一个字符并且所有字符都是字母或数字则返回 True，否则返回 False |
| 11   | [isalpha()](https://www.w3cschool.cn/python3/python3-string-isalpha.html)如果字符串至少有一个字符并且所有字符都是字母或中文字则返回 True, 否则返回 False |
| 12   | [isdigit()](https://www.w3cschool.cn/python3/python3-string-isdigit.html)如果字符串只包含数字则返回 True 否则返回 False |
| 13   | [islower()](https://www.w3cschool.cn/python3/python3-string-islower.html)如果字符串中包含至少一个区分大小写的字符，并且所有这些(区分大小写的)字符都是小写，则返回 True，否则返回 False |
| 14   | [isnumeric()](https://www.w3cschool.cn/python3/python3-string-isnumeric.html)如果字符串中只包含数字字符，则返回 True，否则返回 False |
| 15   | [isspace()](https://www.w3cschool.cn/python3/python3-string-isspace.html)如果字符串中只包含空白，则返回 True，否则返回 False. |
| 16   | [istitle()](https://www.w3cschool.cn/python3/python3-string-istitle.html)如果字符串中所有的单词拼写首字母是否为大写，且其他字母为小写则返回 True，否则返回 False |
| 17   | [isupper()](https://www.w3cschool.cn/python3/python3-string-isupper.html)如果字符串中包含至少一个区分大小写的字符，并且所有这些(区分大小写的)字符都是大写，则返回 True，否则返回 False |
| 18   | [join(seq)](https://www.w3cschool.cn/python3/python3-string-join.html)以指定字符串作为分隔符，将 seq 中所有的元素(的字符串表示)合并为一个新的字符串 |
| 19   | [len(string)](https://www.w3cschool.cn/python3/python3-string-len.html)返回字符串长度 |
| 20   | [ljust(width, fillchar)](https://www.w3cschool.cn/python3/python3-string-ljust.html)返回一个原字符串左对齐,并使用 fillchar 填充至长度 width 的新字符串，fillchar 默认为空格。 |
| 21   | [lower()](https://www.w3cschool.cn/python3/python3-string-lower.html)转换字符串中所有大写字符为小写. |
| 22   | [lstrip()](https://www.w3cschool.cn/python3/python3-string-lstrip.html)截掉字符串左边的空格或指定字符。 |
| 23   | [maketrans()](https://www.w3cschool.cn/python3/python3-string-maketrans.html)创建字符映射的转换表，对于接受两个参数的最简单的调用方式，第一个参数是字符串，表示需要转换的字符，第二个参数也是字符串表示转换的目标。 |
| 24   | [max(str)](https://www.w3cschool.cn/python3/python3-string-max.html)返回字符串 str 中最大的字母。 |
| 25   | [min(str)](https://www.w3cschool.cn/python3/python3-string-min.html)返回字符串 str 中最小的字母。 |
| 26   | [replace(old, new , max)](https://www.w3cschool.cn/python3/python3-string-replace.html)把 将字符串中的 old 替换成 new,如果 max 指定，则替换不超过 max 次。 |
| 27   | [rfind(str, beg=0,end=len(string))](https://www.w3cschool.cn/python3/python3-string-rfind.html)类似于 find()函数，不过是从右边开始查找. |
| 28   | [rindex( str, beg=0, end=len(string))](https://www.w3cschool.cn/python3/python3-string-rindex.html)类似于 index()，不过是从右边开始. |
| 29   | [rjust(width,[, fillchar\])](https://www.w3cschool.cn/python3/python3-string-rjust.html)返回一个原字符串右对齐,并使用fillchar(默认空格）填充至长度 width 的新字符串 |
| 30   | [rstrip()](https://www.w3cschool.cn/python3/python3-string-rstrip.html)删除字符串末尾的空格或指定字符。 |
| 31   | [split(str="", num=string.count(str))](https://www.w3cschool.cn/python3/python3-string-split.html)以 str 为分隔符截取字符串，如果 num 有指定值，则仅截取 num+1 个子字符串 |
| 32   | [splitlines([keepends\])](https://www.w3cschool.cn/python3/python3-string-splitlines.html)按照行('\r', '\r\n', \n')分隔，返回一个包含各行作为元素的列表，如果参数 keepends 为 False，不包含换行符，如果为 True，则保留换行符。 |
| 33   | [startswith(substr, beg=0,end=len(string))](https://www.w3cschool.cn/python3/python3-string-startswith.html)检查字符串是否是以指定子字符串 substr 开头，是则返回 True，否则返回 False。如果beg 和 end 指定值，则在指定范围内检查。 |
| 34   | [strip([chars\])](https://www.w3cschool.cn/python3/python3-string-strip.html)在字符串上执行 lstrip()和 rstrip() |
| 35   | [swapcase()](https://www.w3cschool.cn/python3/python3-string-swapcase.html)将字符串中大写转换为小写，小写转换为大写 |
| 36   | [title()](https://www.w3cschool.cn/python3/python3-string-title.html)返回"标题化"的字符串,就是说所有单词都是以大写开始，其余字母均为小写(见 istitle()) |
| 37   | [translate(table, deletechars="")](https://www.w3cschool.cn/python3/python3-string-translate.html)根据 str 给出的表(包含 256 个字符)转换 string 的字符, 要过滤掉的字符放到 deletechars 参数中 |
| 38   | [upper()](https://www.w3cschool.cn/python3/python3-string-upper.html)转换字符串中的小写字母为大写 |
| 39   | [zfill (width)](https://www.w3cschool.cn/python3/python3-string-zfill.html)返回长度为 width 的字符串，原字符串右对齐，前面填充0 |
| 40   | [isdecimal()](https://www.w3cschool.cn/python3/python3-string-isdecimal.html)检查字符串是否只包含十进制字符，如果是返回 true，否则返回 false。 |



### 列表list相关内置函数

| 方法                                                         | 描述                                                 |
| :----------------------------------------------------------- | :--------------------------------------------------- |
| [append()](https://www.w3school.com.cn/python/ref_list_append.asp) | 在列表的末尾添加一个元素                             |
| [clear()](https://www.w3school.com.cn/python/ref_list_clear.asp) | 删除列表中的所有元素                                 |
| [copy()](https://www.w3school.com.cn/python/ref_list_copy.asp) | 返回列表的副本                                       |
| [count()](https://www.w3school.com.cn/python/ref_list_count.asp) | 返回具有指定值的元素数量。                           |
| [extend()](https://www.w3school.com.cn/python/ref_list_extend.asp) | 将列表元素（或任何可迭代的元素）添加到当前列表的末尾 |
| [index()](https://www.w3school.com.cn/python/ref_list_index.asp) | 返回具有指定值的第一个元素的索引                     |
| [insert()](https://www.w3school.com.cn/python/ref_list_insert.asp) | 在指定位置添加元素                                   |
| [pop()](https://www.w3school.com.cn/python/ref_list_pop.asp) | 删除指定位置的元素                                   |
| [remove()](https://www.w3school.com.cn/python/ref_list_remove.asp) | 删除具有指定值的项目                                 |
| [reverse()](https://www.w3school.com.cn/python/ref_list_reverse.asp) | 颠倒列表的顺序                                       |
| [sort()](https://www.w3school.com.cn/python/ref_list_sort.asp) | 对列表进行排序                                       |

### 集合set相关内置函数

| 方法                                                         | 描述                                         |
| :----------------------------------------------------------- | :------------------------------------------- |
| [add()](https://www.w3school.com.cn/python/ref_set_add.asp)  | 向集合添加元素。                             |
| [clear()](https://www.w3school.com.cn/python/ref_set_clear.asp) | 删除集合中的所有元素。                       |
| [copy()](https://www.w3school.com.cn/python/ref_set_copy.asp) | 返回集合的副本。                             |
| [difference()](https://www.w3school.com.cn/python/ref_set_difference.asp) | 返回包含两个或更多集合之间差异的集合。       |
| [difference_update()](https://www.w3school.com.cn/python/ref_set_difference_update.asp) | 删除此集合中也包含在另一个指定集合中的项目。 |
| [discard()](https://www.w3school.com.cn/python/ref_set_discard.asp) | 删除指定项目。                               |
| [intersection()](https://www.w3school.com.cn/python/ref_set_intersection.asp) | 返回为两个其他集合的交集的集合。             |
| [intersection_update()](https://www.w3school.com.cn/python/ref_set_intersection_update.asp) | 删除此集合中不存在于其他指定集合中的项目。   |
| [isdisjoint()](https://www.w3school.com.cn/python/ref_set_isdisjoint.asp) | 返回两个集合是否有交集。                     |
| [issubset()](https://www.w3school.com.cn/python/ref_set_issubset.asp) | 返回另一个集合是否包含此集合。               |
| [issuperset()](https://www.w3school.com.cn/python/ref_set_issuperset.asp) | 返回此集合是否包含另一个集合。               |
| [pop()](https://www.w3school.com.cn/python/ref_set_pop.asp)  | 从集合中删除一个元素。                       |
| [remove()](https://www.w3school.com.cn/python/ref_set_remove.asp) | 删除指定元素。                               |
| [symmetric_difference()](https://www.w3school.com.cn/python/ref_set_symmetric_difference.asp) | 返回具有两组集合的对称差集的集合。           |
| [symmetric_difference_update()](https://www.w3school.com.cn/python/ref_set_symmetric_difference_update.asp) | 插入此集合和另一个集合的对称差集。           |
| [union()](https://www.w3school.com.cn/python/ref_set_union.asp) | 返回包含集合并集的集合。                     |
| [update()](https://www.w3school.com.cn/python/ref_set_update.asp) | 用此集合和其他集合的并集来更新集合。         |

### 字典dict相关内置函数

| 方法                                                         | 描述                                                   |
| :----------------------------------------------------------- | :----------------------------------------------------- |
| [clear()](https://www.w3school.com.cn/python/ref_dictionary_clear.asp) | 删除字典中的所有元素                                   |
| [copy()](https://www.w3school.com.cn/python/ref_dictionary_copy.asp) | 返回字典的副本                                         |
| [fromkeys()](https://www.w3school.com.cn/python/ref_dictionary_fromkeys.asp) | 返回拥有指定键和值的字典                               |
| [get()](https://www.w3school.com.cn/python/ref_dictionary_get.asp) | 返回指定键的值                                         |
| [items()](https://www.w3school.com.cn/python/ref_dictionary_items.asp) | 返回包含每个键值对的元组的列表                         |
| [keys()](https://www.w3school.com.cn/python/ref_dictionary_keys.asp) | 返回包含字典键的列表                                   |
| [pop()](https://www.w3school.com.cn/python/ref_dictionary_pop.asp) | 删除拥有指定键的元素                                   |
| [popitem()](https://www.w3school.com.cn/python/ref_dictionary_popitem.asp) | 删除最后插入的键值对                                   |
| [setdefault()](https://www.w3school.com.cn/python/ref_dictionary_setdefault.asp) | 返回指定键的值。如果该键不存在，则插入具有指定值的键。 |
| [update()](https://www.w3school.com.cn/python/ref_dictionary_update.asp) | 使用指定的键值对字典进行更新                           |
| [values()](https://www.w3school.com.cn/python/ref_dictionary_values.asp) | 返回字典中所有值的列表                                 |
